/* File: DateTest.java
 * Contains: Main program for time and date formatting
 * Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Created: March 1998
 * Updated: Thu Mar 19 18:11:27 1998 by Jeff Dalton
 * Copyright: (c) 1998, AIAI, University of Edinburgh
 */

package ix.test;

import java.util.*;

import java.text.DateFormat;
import java.text.SimpleDateFormat;


/** A class for testing */

class DateTest {

    static final int[] style
       = {DateFormat.DEFAULT, DateFormat.FULL, DateFormat.LONG,
	  DateFormat.MEDIUM, DateFormat.SHORT};

    static final String [] style_name
       = {"DEFAULT", "FULL   ", "LONG   ", "MEDIUM ", "SHORT  "};

    static String datePattern = "dd-MMM-yy"; // e.g. 18-Mar-98

    static String timePattern = "HH:mm:ss";  // e.g. 21:51:05 for 9:51:05 PM

    public static void main(String[] argv) {

	// Now

	Date d = new Date();
	TimeZone tz = TimeZone.getDefault();

	// Show what various standard formats look like.

	for (int i = 0; i < style.length; i++) {
	    DateFormat df = DateFormat.getDateTimeInstance(style[i], style[i]);
	    System.out.println(style_name[i] + ": " + df.format(d));
	}

	// Our own format

	System.out.println("");
	System.out.println
	    ("Using format patterns " + datePattern
	     + " and " + timePattern + ":");

	SimpleDateFormat sdf
	    = new SimpleDateFormat(datePattern + "-" + timePattern);

	System.out.println(sdf.format(d));

	// Maybe we have to explicitly set the default TimeZone?  /\/
	// It matters with JDK 1.1.5 on the AIAI Suns, but not
	// on PCs running FreeBSD.

	System.out.println("\nInstalling default TimeZone...\n");
	sdf.setTimeZone(tz);

	System.out.println(sdf.format(d));

    }

}
