/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Fri Sep 15 22:14:36 2000 by Jeff Dalton
 */

package ix.util.lisp;

import java.util.*;
import ix.util.*;

/**
 * A Collector that uses an LList to hold the elements and that provides
 * a number of different ways to add elements.
 */

public class ListCollector implements Collector {

    Cons head = new Cons(Lisp.NIL, Lisp.NIL);
    Cons tail = head;

    public ListCollector() {
    }

    public void addElement(Object e) {
	tail.cdr = new Cons(e, Lisp.NIL);
	tail = (Cons)tail.cdr;
    }

    public void deleteElement(Object e) {
	setContents(head.cdr.delete(e));
    }

    public void pushElement(Object e) {
	head.cdr = new Cons(e, head.cdr);
	if (tail == head)
	    tail = (Cons)tail.cdr;
    }

    public Object popElement() {
	Debug.assert(head.cdr instanceof Cons, "popElement() when empty");
	Object result = (head.cdr).car();
	if (tail == head.cdr) {		// popping final element
	    Debug.assert(tail.cdr == Lisp.NIL, "bogus tail");
	    tail = head;
	}
	head.cdr = ((Cons)head.cdr).cdr;
	return result;
    }

    public Enumeration elements() {
	return head.cdr.elements();
    }

    public LList contents() {
	return head.cdr;
    }

    public void setContents(LList newContents) {
	head.cdr = newContents;
	tail = head;
	if (newContents instanceof Cons)
	    tail = tail.lastCons();
    }

    public boolean isEmpty() {
	return (tail == head);
    }

    public int length() {
	return (head.cdr).length();
    }

    public void concList(LList l) {
	tail.cdr = l;
	if (l instanceof Cons)
	    tail = tail.lastCons();
    }

    public void appendList(LList l) {
	concList((LList)l.clone());
    }

    public void pushList(LList l) { 		// prefixList()?  /\/
	setContents(l.append(head.cdr));
    }

    public Function1 elementAdder() {
	return new adderFunction() {
	    public Object funcall(Object e) {
		addElement(e);
		return null;
	    }
	};
    }

    public Function1 elementPusher() {
	return new adderFunction() {
	    public Object funcall(Object e) {
		pushElement(e);
		return null;
	    }
	};
    }

    public Function1 listAppender() {
	return new adderFunction() {
	    public Object funcall(Object l) {
		appendList((LList)l);
		return null;
	    }
	};
    }

    public Function1 listConcer() {
	return new adderFunction() {
	    public Object funcall(Object l) {
		concList((LList)l);
		return null;
	    }
	};
    }

    protected static interface adderFunction extends Function1 {
    }

}
